// CDXTileAnim.cpp: implementation of the CDXTileAnim class.
//
//////////////////////////////////////////////////////////////////////

#include "CDXTileAnim.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDXTileAnim::CDXTileAnim()
{
	Init();
}

CDXTileAnim::CDXTileAnim(CDXSprite* tileSpr, int height, WORD bltType)
	: CDXTileUnit(tileSpr, 0, height, bltType)
{
	Init();
}

CDXTileAnim::~CDXTileAnim()
{

}

void CDXTileAnim::AddFrame(DWORD frame, DWORD delay)
{
	// First frame, init 
	if (m_AnimFrames.GetSize()==0)
	{
		SetFrame(frame);
		m_FrameDelay = delay;
	}
	m_AnimFrames.Add(frame);
	m_AnimDelays.Add(delay);
}

int CDXTileAnim::GetFrameCount()
{
	return m_AnimFrames.GetSize();
}

void CDXTileAnim::UpdateAnim()
{
	if (m_FrameDelay-- < 0)
	{
		// next frame
		Assert(GetFrameCount()>0, "CDXTileAnim::UpdateAnim\nEmpty Frame");
		if (++m_FrameIndex > m_AnimFrames.GetUpperBound())
			m_FrameIndex = 0;

		// Could just call SetFrameIndex
		SetFrame(m_AnimFrames[m_FrameIndex]);
		m_FrameDelay = m_AnimDelays[m_FrameIndex];
	}
}

void CDXTileAnim::Init()
{
	m_FrameDelay = 0;
	m_FrameIndex = 0;
}

int CDXTileAnim::RandomAnim()
{
	Assert(GetFrameCount()>0, "CDXTileAnim::RandomAnim\nEmpty Frame");
	int oldFrameIndex = m_FrameIndex;
	SetFrameIndex(rand()%m_AnimFrames.GetSize());
	return oldFrameIndex;
}

void CDXTileAnim::SetFrameIndex(int frameIndex)
{
	Assert(frameIndex<GetFrameCount(), "CDXTileAnim::SetFrameIndex\nFrame index out of range");
	m_FrameIndex = frameIndex;
	SetFrame(m_AnimFrames[m_FrameIndex]);
	m_FrameDelay = m_AnimDelays[m_FrameIndex];
}
