// CDXTileEngine.cpp: implementation of the CDXTileEngine class.
//
//////////////////////////////////////////////////////////////////////

#include "CDXTileViewer.h"
#include "CDXTileEngine.h"
#include "CDXTileLayer.h"
#include "CDXTileStack.h"
#include "CDXTileUnit.h"
#include "CDXTileAnim.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDXTileEngine::CDXTileEngine()
{
	Init();
}

CDXTileEngine::~CDXTileEngine()
{
	// under development
}

void CDXTileEngine::SetULPos(int x, int y)
{
	// Every Pos Setting Function will always call this
	// function at the end
	m_PosX = x; m_PosY = y;
}	

void CDXTileEngine::SetMapPos(int x, int y)
{
	SetULPos(x,y);
}

CDXTileLayer* CDXTileEngine::CreateLayer(int mapDimX, int mapDimY, int tileSizeX, int tileSizeY, int layerIndex)
{
	CDXTileLayer* pLayer = 
		Register(0, new CDXTileLayer(this, mapDimX, mapDimY, tileSizeX, tileSizeY));
	SetLayer(pLayer, layerIndex);	
	return pLayer;
}

BOOL CDXTileEngine::SaveFile(char *filename)
{
	// under development
	return true;
}

BOOL CDXTileEngine::LoadFile(char *filename)
{
	// under development	
	return true;
}

void CDXTileEngine::Draw()
{
	Assert(GetViewer() != 0, "CDXTileEngine::Draw\nNo Viewer");
	GetViewer()->Draw(this);
}

int CDXTileEngine::GetLayerCount()
{
	return m_Layers.GetSize();
}

CDXTileLayer* CDXTileEngine::GetLayer(int index)
{
	Assert(index < GetLayerCount(), "CDXTileEngine::GetLayer\nLayer index out of range");
	return (CDXTileLayer*)m_Layers[index];
}

void CDXTileEngine::Init()
{
	m_PosX = 0; m_PosY = 0;
	m_pViewer = 0;
}

void CDXTileEngine::GetMapPos(int *x, int *y)
{
	*x = m_PosX;
	*y = m_PosY;
}

CDXTileViewer* CDXTileEngine::SetViewer(CDXTileViewer *pViewer)
{
	CDXTileViewer* pOldViewer = m_pViewer;
	m_pViewer = pViewer;
	return pOldViewer;
}

CDXTileViewer* CDXTileEngine::GetViewer()
{
	return m_pViewer;
}

void CDXTileEngine::ScrollMap(int distPosX, int distPosY)
{
	SetULPos(m_PosX+distPosX, m_PosY+distPosY);
}

void CDXTileEngine::SetLayer(CDXTileLayer *pLayer, int layerIndex)
{
	if (layerIndex == NEW_LAYER)
		layerIndex = GetLayerCount(); else
	if (layerIndex == LAST_LAYER)
		layerIndex = GetLayerCount()-1;

	while (GetLayerCount() <= layerIndex)
		m_Layers.Add(NULL);	// insert empty layer

	m_Layers[layerIndex] = pLayer;
}

CDXTileManager* CDXTileEngine::GetManager()
{
	return 0;//???&m_Manager;
}

CDXTileStack* CDXTileEngine::CreateTileStack()
{
	return Register(0, new CDXTileStack);
}

CDXTileUnit* CDXTileEngine::CreateTileUnit(CDXSprite *tileSpr, int frame, int height, WORD bltType)
{
	return Register(0, new CDXTileUnit(tileSpr, frame, height, bltType));
}

CDXTileAnim* CDXTileEngine::CreateTileAnim(CDXSprite *tileSpr, int height, WORD bltType)
{
	return Register(0, new CDXTileAnim(tileSpr, height, bltType));
}

void CDXTileEngine::UpdateAnim()
{
	int count = GetTileAnimCount();
	for (int i=0; i<count; i++)
		GetTileAnim(i)->UpdateAnim();
}

int CDXTileEngine::FindLayer(CDXTileLayer *pLayer)
{
	int count = GetLayerCount();
	for (int i=0; i<count; i++)
		if (GetLayer(i)==pLayer)
			return i;
	return -1;
}

void CDXTileEngine::DeleteLayer(int layerIndex)
{
	Assert(layerIndex >= 0 && layerIndex < GetLayerCount(), "CDXTileEngine::DeleteLayer\nLayer index of out range");
	DeleteTileLayer(
		Find(GetLayer(layerIndex)));
	m_Layers.RemoveAt(layerIndex);
}

void CDXTileEngine::RandomAnim()
{
	int count = GetTileAnimCount();
	for (int i=0; i<count; i++)
		GetTileAnim(i)->RandomAnim();
}
