// CDXTileIsoViewer.cpp: implementation of the CDXTileIsoViewer class.
//
//////////////////////////////////////////////////////////////////////

#include "CDXTileIsoViewer.h"
#include "CDXTileSpriteBase.h"
#include "CDXTileIsoViewer.h"
#include "CDXTileEngine.h"
#include "CDXTileLayer.h"
#include "CDXTileStack.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CDXTileIsoViewer::CDXTileIsoViewer()
{

}

CDXTileIsoViewer::CDXTileIsoViewer(CDXSurface* pSurface)
: CDXTileViewer(pSurface)
{

}

CDXTileIsoViewer::~CDXTileIsoViewer()
{

}

void CDXTileIsoViewer::Draw(CDXTileLayer *pLayer)
{
	// Note : This viewer currently can only be used
	// to view tile with ratio of width:height = 2:1 
	// I will fix this soon after I've found the right formula
	Assert(m_pSurface, "CDXTileIsoViewer::Draw\nNo surface");
	
	int posX, posY;
	pLayer->GetEngine()->GetMapPos(&posX, &posY);

	// Get Layer Capabilities
	BOOL bTileCap = pLayer->GetCapabilities(CDXTileLayer::CAP_TILE);
	BOOL bSpriteCap = pLayer->GetCapabilities(CDXTileLayer::CAP_SPRITE);
	
	RECT viewRect, oldClipRect;
	BOOL bClipped = GetViewRect(viewRect);
	if (bClipped)
	{
		// Clip the Screen
		m_pSurface->GetClipRect(&oldClipRect);
		m_pSurface->SetClipRect(&viewRect);
	} else
	{
		viewRect.left = 0;
		viewRect.top = 0;
		viewRect.right = m_pSurface->m_PixelWidth;
		viewRect.bottom = m_pSurface->m_PixelHeight;
	}

	int viewSizeX = viewRect.right-viewRect.left;
	int viewSizeY = viewRect.bottom-viewRect.top;

	// Get Various Info
	int tileSizeX = pLayer->m_TileSizeX;
	int tileSizeY = pLayer->m_TileSizeY;
	int mapDimX = pLayer->m_MapDimX;
	int mapDimY = pLayer->m_MapDimY;
	int mapSizeX = mapDimX*tileSizeX;
	int mapSizeY = mapDimY*tileSizeY;

	int halfTileSizeX = tileSizeX >> 1;
	int halfTileSizeY = tileSizeY >> 1;

	// Wrap position
	while (posX < 0)
		posX += mapSizeX;
	while (posY < 0)
		posY += mapSizeY;
	while (posX >= mapSizeX)
		posX -= mapSizeX;
	while (posY >= mapSizeY)
		posY -= mapSizeY;

	// For Debugging
	/*CString temp;
	temp.Format("%d , %d", posX,posY);
	m_pSurface->GetDC();
	m_pSurface->TextXY(500, 100, 0xFFFFFF, temp);
	m_pSurface->ReleaseDC();*/

	int coordX = posX / tileSizeX -
		pLayer->m_VisibleTileBonus.left;
	int offsetX = posX % tileSizeX;
	int coordY = posY / tileSizeY -
		pLayer->m_VisibleTileBonus.top;
	int offsetY = posY % tileSizeY;

	// New
	if (coordX >= mapDimX)
		coordX -= mapDimX;
	if (coordX < 0)
		coordX += mapDimX;
	if (coordY >= mapDimY)
		coordY -= mapDimY;
	if (coordY < 0)
		coordY += mapDimY;

	int x_add = offsetX *tileSizeY/tileSizeX  - offsetY;
	int y_add = ((offsetX *tileSizeY/tileSizeX)  >> 1) + (offsetY >> 1);

	int screenY = halfTileSizeY-y_add+viewRect.top -
		pLayer->m_VisibleTileBonus.top*halfTileSizeY;

	// you would need a greater m_VisibleY
	// when using very high objects
	const int m_VisibleY = 
		viewSizeY/halfTileSizeY+3+
		pLayer->m_VisibleTileBonus.top+pLayer->m_VisibleTileBonus.bottom;
	const int m_VisibleX = viewSizeX/tileSizeX+2+
		pLayer->m_VisibleTileBonus.left+pLayer->m_VisibleTileBonus.right;

    for(int i=0; i<m_VisibleY; i++)
	{
		int tempX = coordX;
		int tempY = coordY;

		int screenX = -halfTileSizeX-x_add+viewRect.left -
			pLayer->m_VisibleTileBonus.left*tileSizeX;

		int RealVisibleX = m_VisibleX;
		if (i & 1)
		{
			RealVisibleX++;
			screenX -= halfTileSizeX;
		}

		for (int j=0; j<RealVisibleX; j++)
		{
			if (bTileCap)
			{
				CDXTileStack* pStack = pLayer->GetTileStack(tempX, tempY);
				if (pStack)
					pStack->Draw(m_pSurface, screenX, screenY);
			}

			if (bSpriteCap)
			{
				CPtrArray* pSpriteArray = pLayer->GetSpriteArray(tempX, tempY);
				int spriteCount;
				if (spriteCount = pSpriteArray->GetSize())
					for (int k=0; k<spriteCount; k++)
					{
						CDXTileSpriteBase* pSprite =
							(CDXTileSpriteBase*)pSpriteArray->GetAt(k);
						int spriteXOff = pSprite->GetPosX() % tileSizeX;
						int spriteYOff = pSprite->GetPosY() % tileSizeY;
						// there are minus tileSizeX dan tileSizeY
						pSprite->OnDraw(m_pSurface,
							screenX + (spriteXOff*tileSizeY/tileSizeX - spriteYOff),// - tileSizeX,
							screenY + (((spriteXOff *tileSizeY/tileSizeX) >> 1) + (spriteYOff >> 1)));// - tileSizeY);
					}
			}


			
			/*CString temp;
			temp.Format("%d", tempX);
			m_pSurface->GetDC();
			m_pSurface->TextXY(screenX, screenY, 0xFFFFFF, temp);
			m_pSurface->ReleaseDC();*/

			screenX += tileSizeX;
			
			// map update and wrap
			if (++tempX >= mapDimX)
				tempX = 0;
			if (--tempY < 0)
				tempY = mapDimY-1;
		}
	
		screenY += halfTileSizeY;
		if (i & 1)
		{
			if (++coordX >= mapDimX)
				coordX = 0;
		} else
		{
			if (++coordY >= mapDimY)
				coordY = 0;
		}

	}
	// Return the original ClipRect
	if (bClipped)
		m_pSurface->SetClipRect(&oldClipRect);
}
