// CDXTileLayer.h: interface for the CDXTileLayer class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CDXTILELAYER_H__38C8C782_FCBB_11CF_A2B6_9AB547EC654B__INCLUDED_)
#define AFX_CDXTILELAYER_H__38C8C782_FCBB_11CF_A2B6_9AB547EC654B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "CDXTileBase.h"
#include "CDXTileSpriteBase.h"
#include "CDXTileManager.h"
#include "CDXTileIsoViewer.h"

class CDXTileEngine;
class CDXTileStack;

class CDXTileLayer : public CDXTileBase 
{
friend void CDXTileIsoViewer::Draw(CDXTileLayer*);
friend CDXTileLayer* CDXTileManager::Register(const char* name, CDXTileLayer*);
friend void CDXTileSpriteBase::SetPos(int posX, int posY);

// Capabilities
public:
enum {CAP_TILE = 1, CAP_SPRITE = 2};

public:
	void SetVisibleTileBonus(int leftTile, int topTile, int rightTile, int bottomTile);
	void SetCapability(DWORD capability, BOOL set = TRUE);
	BOOL GetCapabilities(DWORD capabilities);
	void SetCapabilities(DWORD capabilities);
	/*void GetCapabilities(BOOL& bTileCap, BOOL& bSpriteCap);
	void SetCapabilities(BOOL bTileContainer, BOOL bSpriteContainer);*/
	CPtrArray* GetSpriteArray(int coordX, int coordY);
	void UpdateSprite(CDXTileSpriteBase* pSprite, int oldCoordX, int oldCoordY, int& spriteIndex, int newCoordX, int newCoordY);
	void PosToCoord(int posX, int posY, int& coordX, int& coordY);
	void GetTileSize(int& tileSizeX, int& tileSizeY);
	void SetWrapFlag(bool bWrap);
	CDXTileEngine* GetEngine();
	CDXTileViewer* GetActiveViewer();
	CDXTileViewer* GetViewer();
	CDXTileViewer* SetViewer(CDXTileViewer* pViewer);
	CDXTileStack* GetTileStack(int coordX, int coordY);
	int GetTileStackCount();
	void ClearMap();
	void SetTileSize(int tileSizeX, int tileSizeY);
	void SetMapDimension(int mapDimX, int mapDimY);
	void SetTileStack(int x, int y, CDXTileStack* tileStack);
	BOOL Draw();
	CDXTileLayer(CDXTileEngine* pEngine);
	CDXTileLayer(CDXTileEngine* pEngine, int mapDimX, int mapDimY, int tileSizeX, int tileSizeY);
	virtual ~CDXTileLayer();

	BOOL SaveFile(char* filename);
	BOOL LoadFile(char* filename);

protected: // it's should be protected
	void Init(CDXTileEngine* pEngine);
	int m_PosXDiv, m_PosYDiv;	// divider of map position, usefull for creating parallax effect
	CDXTileEngine* m_pEngine;	// pointer to the engine
	CDXTileStack** m_TileStacks;  // array 2 dimension that point to TileStack
	CPtrArray* m_Sprites;		// array 2 dimension that point to array of Sprite
	CDXTileViewer* m_pViewer;	// specific viewer for this class
	int m_MapDimX, m_MapDimY;	// Map Dimension
	int m_TileSizeX, m_TileSizeY;	// Tile Size
	RECT m_VisibleTileBonus;	// bonus tile to draw for 4 direction
};

#endif // !defined(AFX_CDXTILELAYER_H__38C8C782_FCBB_11CF_A2B6_9AB547EC654B__INCLUDED_)
