// CDXTileManager.cpp: implementation of the CDXTileManager class.
//
//////////////////////////////////////////////////////////////////////

#include "CDXTileManager.h"
#include "CDXTileViewer.h"
#include "CDXTileLayer.h"
#include "CDXTileStack.h"
#include "CDXTileUnit.h"
#include "CDXTileAnim.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDXTileManager::CDXTileManager()
{

}

CDXTileManager::~CDXTileManager()
{
	// Destroy All
	int count = GetTileUnitCount();
	for (int i=0; i<count; i++)
		delete GetTileUnit(i);

	count = GetTileAnimCount();
	for (i=0; i<count; i++)
		delete GetTileAnim(i);
	
	count = GetTileStackCount();
	for (i=0; i<count; i++)
		delete GetTileStack(i);

	count = GetTileLayerCount();
	for (i=0; i<count; i++)
		delete GetTileLayer(i);

	count = GetTileViewerCount();
	for (i=0; i<count; i++)
		delete GetTileViewer(i);

	count = m_CDXTiles.GetSize();
	for (i=0; i<count; i++)
		delete (CDXTile*)m_CDXTiles[i];

	count = m_CDXSprites.GetSize();
	for (i=0; i<count; i++)
		delete (CDXSprite*)m_CDXSprites[i];
}

CDXTileUnit* CDXTileManager::Register(const char* name, CDXTileUnit *pUnit)
{
	Assert(pUnit, "CDXTileManager::Register\nEmpty TileUnit");
	
	// Can't register twice for different pointer
#ifdef _DEBUG
	if (name && m_TileUnitMap[name])
	{
		Assert(m_TileUnitMap[name] == pUnit, "CDXTileManager::Register\nTileUnit already registered using different pointer");
	}
#endif
	
	// Name can be map more than once, 
	// but the pointer won't be registered again
	if (name)
		m_TileUnitMap[name] = pUnit;
	
	// Don't register twice
	if (pUnit->GetFlag(BIT_REGISTER))
		return pUnit;

	pUnit->SetFlag(BIT_REGISTER);

	/*if (pUnit->m_InternalFlags & BIT_REGISTER)
		return pUnit;
	pUnit->m_InternalFlags |= BIT_REGISTER;*/

	m_TileUnits.Add(pUnit);	
	return pUnit;
}

CDXTileAnim* CDXTileManager::Register(const char* name, CDXTileAnim *pAnim)
{
	Assert(pAnim, "CDXTileManager::Register\nEmpty TileAnim");
	
	// Can't register twice for different pointer
#ifdef _DEBUG
	if (name && m_TileAnimMap[name])
	{
		Assert(m_TileAnimMap[name] == pAnim, "CDXTileManager::Register\nTileAnim already registered using different pointer");
	}
#endif
	
	// Name can be map more than once, 
	// but the pointer won't be registered again
	if (name)
		m_TileAnimMap[name] = pAnim;

	// Don't register twice
	if (pAnim->GetFlag(BIT_REGISTER))
		return pAnim;
	pAnim->SetFlag(BIT_REGISTER);
	m_TileAnims.Add(pAnim);	
	return pAnim;
}


CDXTileStack* CDXTileManager::Register(const char* name, CDXTileStack *pStack)
{
	Assert(pStack, "CDXTileManager::Register\nEmpty TileStack");
	
	// Can't register twice for different pointer
#ifdef _DEBUG
	if (name && m_TileStackMap[name])
	{
		Assert(m_TileStackMap[name] == pStack, "CDXTileManager::Register\nTileStack already registered using different pointer");
	}
#endif
	
	// Name can be map more than once, 
	// but the pointer won't be registered again
	if (name)
		m_TileStackMap[name] = pStack;

	// Don't register twice
	if (pStack->GetFlag(BIT_REGISTER))
		return pStack;
	pStack->SetFlag(BIT_REGISTER);
	m_TileStacks.Add(pStack);	
	return pStack;
}

CDXTileLayer* CDXTileManager::Register(const char* name, CDXTileLayer *pLayer)
{
	Assert(pLayer, "CDXTileManager::Register\nEmpty TileLayer");
	
	// Can't register twice for different pointer
#ifdef _DEBUG
	if (name && m_TileLayerMap[name])
	{
		Assert(m_TileLayerMap[name] == pLayer, "CDXTileManager::Register\nTileLayer already registered using different pointer");
	}
#endif
	
	// Name can be map more than once, 
	// but the pointer won't be registered again
	if (name)
		m_TileLayerMap[name] = pLayer;

	// Don't register twice
	if (pLayer->GetFlag(BIT_REGISTER))
		return pLayer;
	pLayer->SetFlag(BIT_REGISTER);
	m_TileLayers.Add(pLayer);	
	return pLayer;
}

/*CDXTileEngine* CDXTileManager::Register(CDXTileEngine *pEngine)
{
	// Don't register twice
	if (pEngine->GetFlag(BIT_REGISTER))
		return pEngine;
	pEngine->SetFlag(BIT_REGISTER);
	m_TileEngines.Add(pEngine);	
	return pEngine;
}*/

CDXTileUnit* CDXTileManager::GetTileUnit(int index)
{
	ASSERT(index < GetTileUnitCount());
	return (CDXTileUnit*)m_TileUnits[index];
}

int CDXTileManager::GetTileUnitCount()
{
	return m_TileUnits.GetSize();
}

CDXTileStack* CDXTileManager::GetTileStack(int index)
{
	ASSERT(index < GetTileStackCount());
	return (CDXTileStack*)m_TileStacks[index];
}

int CDXTileManager::GetTileStackCount()
{
	return m_TileStacks.GetSize();
}

CDXTileLayer* CDXTileManager::GetTileLayer(int index)
{
	ASSERT(index < GetTileLayerCount());
	return (CDXTileLayer*)m_TileLayers[index];
}

int CDXTileManager::GetTileLayerCount()
{
	return m_TileLayers.GetSize();
}

CDXTileViewer* CDXTileManager::Register(const char* name, CDXTileViewer *pViewer)
{
	Assert(pViewer, "CDXTileManager::Register\nEmpty TileViewer");
	
	// Can't register twice for different pointer
#ifdef _DEBUG
	if (name && m_TileViewerMap[name])
	{
		Assert(m_TileViewerMap[name] == pViewer, "CDXTileManager::Register\nTileViewer already registered using different pointer");
	}
#endif
	
	// Name can be map more than once, 
	// but the pointer won't be registered again
	if (name)
		m_TileViewerMap[name] = pViewer;

	// Don't register twice
	if (pViewer->GetFlag(BIT_REGISTER))
		return pViewer;
	pViewer->SetFlag(BIT_REGISTER);
	m_TileViewers.Add(pViewer);	
	return pViewer;
}

int CDXTileManager::GetTileViewerCount()
{
	return m_TileViewers.GetSize();
}

CDXTileViewer* CDXTileManager::GetTileViewer(int index)
{
	ASSERT(index < GetTileViewerCount());	
	return (CDXTileViewer*)m_TileViewers[index];
}

int CDXTileManager::GetTileAnimCount()
{
	return m_TileAnims.GetSize();
}

CDXTileAnim* CDXTileManager::GetTileAnim(int index)
{
	ASSERT(index < GetTileAnimCount());
	return (CDXTileAnim*)m_TileAnims[index];
}

int CDXTileManager::Find(CDXTileLayer *pLayer)
{
	int count = m_TileLayers.GetSize();
	for (int i=0; i<count; i++)
		if (GetTileLayer(i)==pLayer)
			return i;
	return -1;
}

void CDXTileManager::DeleteTileLayer(int index)
{
	ASSERT(index < GetTileLayerCount());
	delete GetTileLayer(index);
	m_TileLayers.RemoveAt(index);
}

/*int CDXTileManager::Find(CDXTileEngine *pEngine)
{
	int count = m_TileEngines.GetSize();
	for (int i=0; i<count; i++)
		if (GetTileEngine(i)==pEngine)
			return i;
	return -1;

}*/

int CDXTileManager::Find(CDXTileStack *pStack)
{
	int count = m_TileStacks.GetSize();
	for (int i=0; i<count; i++)
		if (GetTileStack(i)==pStack)
			return i;
	return -1;

}

int CDXTileManager::Find(CDXTileUnit *pUnit)
{
	int count = m_TileUnits.GetSize();
	for (int i=0; i<count; i++)
		if (GetTileUnit(i)==pUnit)
			return i;
	return -1;
}

int CDXTileManager::Find(CDXTileViewer *pViewer)
{
	int count = m_TileViewers.GetSize();
	for (int i=0; i<count; i++)
		if (GetTileViewer(i)==pViewer)
			return i;
	return -1;
}

int CDXTileManager::Find(CDXTileAnim *pAnim)
{
	int count = m_TileAnims.GetSize();
	for (int i=0; i<count; i++)
		if (GetTileAnim(i)==pAnim)
			return i;
	return -1;
}

void CDXTileManager::DeleteTileViewer(int index)
{
	ASSERT(index < GetTileViewerCount());
	delete GetTileViewer(index);
	m_TileViewers.RemoveAt(index);
}

void CDXTileManager::DeleteTileStack(int index)
{
	ASSERT(index < GetTileStackCount());
	delete GetTileStack(index);
	m_TileStacks.RemoveAt(index);
}

void CDXTileManager::DeleteTileUnit(int index)
{
	ASSERT(index < GetTileUnitCount());
	delete GetTileUnit(index);
	m_TileUnits.RemoveAt(index);
}

void CDXTileManager::DeleteTileAnim(int index)
{
	ASSERT(index < GetTileAnimCount());
	delete GetTileAnim(index);
	m_TileAnims.RemoveAt(index);
}

CDXTile* CDXTileManager::Tile(const char* name)
{
	Assert(name, "CDXTileManager::Tile()\nEmpty name not allowed");
	Assert2(name && m_CDXTileMap[name], "CDXTileManager::Tile()\nUnknown tile name : ", name);
	return (CDXTile*)m_CDXTileMap[name];
}

CDXSprite* CDXTileManager::Sprite(const char* name)
{
	Assert(name, "CDXTileManager::Sprite()\nEmpty name not allowed");
	Assert2(name && m_CDXSpriteMap[name], "CDXTileManager::Sprite()\nUnknown sprite name : ", name);
	return (CDXSprite*)m_CDXSpriteMap[name];
}

CDXTileLayer* CDXTileManager::TileLayer(const char* name)
{
	Assert(name, "CDXTileManager::TileLayer()\nEmpty name not allowed");
	Assert2(name && m_TileLayerMap[name], "CDXTileManager::TileLayer()\nUnknown TileLayer name : ", name);
	return (CDXTileLayer*)m_TileLayerMap[name];
}

CDXTileAnim* CDXTileManager::TileAnim(const char* name)
{
	Assert(name, "CDXTileManager::TileAnim()\nEmpty name not allowed");
	Assert2(name && m_TileAnimMap[name], "CDXTileManager::TileAnim()\nUnknown TileAnim name : ", name);
	return (CDXTileAnim*)m_TileAnimMap[name];
}

CDXTileViewer* CDXTileManager::TileViewer(const char* name)
{
	Assert(name, "CDXTileManager::TileViewer()\nEmpty name not allowed");
	Assert2(name && m_TileViewerMap[name], "CDXTileManager::TileViewer()\nUnknown TileViewer name : ", name);
	return (CDXTileViewer*)m_TileViewerMap[name];
}

CDXTileUnit* CDXTileManager::TileUnit(const char* name)
{
	Assert(name, "CDXTileManager::TileUnit()\nEmpty name not allowed");
	Assert2(name && m_TileUnitMap[name], "CDXTileManager::TileUnit()\nUnknown TileUnit name : ", name);
	return (CDXTileUnit*)m_TileUnitMap[name];
}

CDXTileStack* CDXTileManager::TileStack(const char* name)
{
	Assert(name, "CDXTileManager::TileStack()\nEmpty name not allowed");
	Assert2(name && m_TileStackMap[name], "CDXTileManager::TileStack()\nUnknown TileStack name : ", name);
	return (CDXTileStack*)m_TileStackMap[name];
}

CDXTile* CDXTileManager::Register(const char *name, CDXTile *pTile)
{
	if (name)
		m_CDXTileMap[name] = pTile;
	m_CDXTiles.Add(pTile);	
	return pTile;
}

CDXSprite* CDXTileManager::Register(const char *name, CDXSprite *pSprite)
{
	if (name)
		m_CDXSpriteMap[name] = pSprite;
	m_CDXSprites.Add(pSprite);
	return pSprite;
}

void CDXTileManager::RestoreAll()
{
	ReleaseAll();
	int count = m_CDXTiles.GetSize();
	for (int i=0; i<count; i++)
	{
		CDXTile* pTile = (CDXTile*)m_CDXTiles[i];

		// The command to restore a CDXSurface should be pTile->Restore()
		// But, it doesn't work, it failed in DDCopyBitmap when calling GetDC, don't know why?
		// So I've just to use other way, for temporary only.

		pTile->m_lpDDS = DDLoadSizeBitmap(
			pTile->Screen->m_lpDD, 
			pTile->m_pFilename, 
			&pTile->m_PixelWidth, 
			&pTile->m_PixelHeight,
			CDXMEM_VIDTHENSYS);
		pTile->SetColorKey(0);
	}
}

void CDXTileManager::ReleaseAll()
{
	int count = m_CDXTiles.GetSize();
	for (int i=0; i<count; i++)
	{
		CDXTile* pTile = (CDXTile*)m_CDXTiles[i];
		RELEASE(pTile->m_lpDDS);
	}
}
