// CDXTileManager.h: interface for the CDXTileManager class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CDXTILEMANAGER_H__3DD28F41_FCC5_11CF_A2B6_82A0EDBA634B__INCLUDED_)
#define AFX_CDXTILEMANAGER_H__3DD28F41_FCC5_11CF_A2B6_82A0EDBA634B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "CDXTileBase.h"

class CDXTileViewer;
class CDXTileEngine;
class CDXTileLayer;
class CDXTileStack;
class CDXTileUnit;
class CDXTileAnim;

class CDXTileManager : public CDXTileBase
{
public:
	void ReleaseAll();
	void RestoreAll();
	CDXTile* CreateTile(const char* name)
		{ return Register(name, new CDXTile); }
	CDXTile* CreateTile(const char* name, CDXScreen *pScreen, const char* szFilename, int w, int h, int num, BOOL memoryType = CDXMEM_VIDTHENSYS)
		{ return Register(name, new CDXTile(pScreen, szFilename, w, h, num, memoryType)); }

	CDXSprite* CreateSprite(const char* name)
		{ return Register(name, new CDXSprite); }
	CDXSprite* CreateSprite(const char* name, CDXTile* pTile)
		{ return Register(name, new CDXSprite(pTile)); }
	CDXSprite* CreateSprite(const char* name, CDXScreen *pScreen, const char* Filename, int w, int h, int num, BOOL memoryType)
		{ return Register(name, new CDXSprite(pScreen, Filename, w, h, num, memoryType)); }

	CDXTileLayer* Register(const char* name, CDXTileLayer* pLayer);
	CDXTileAnim* Register(const char* name, CDXTileAnim* pAnim);
	CDXTileViewer* Register(const char* name, CDXTileViewer* pViewer);
	CDXTileUnit* Register(const char* name, CDXTileUnit* pUnit);
	CDXTileStack* Register(const char* name, CDXTileStack* pStack);
	//CDXTile* Register(char* name, CDXTile* pTile);
	//CDXSprite* Register(char* name, CDXSprite* pSprite);
	//CDXTileEngine* Register(char* name = 0, CDXTileEngine* pEngine = 0);

	CDXTile* Tile(const char* name);
	
	CDXSprite* Sprite(const char* name);
	CDXTileLayer* TileLayer(const char* name);
	CDXTileAnim* TileAnim(const char* name);
	CDXTileViewer* TileViewer(const char* name);
	CDXTileUnit* TileUnit(const char* name);
	CDXTileStack* TileStack(const char* name);

	void DeleteTileAnim(int index);
	void DeleteTileUnit(int index);
	void DeleteTileStack(int index);
	//void DeleteTileEngine(int index);
	void DeleteTileViewer(int index);
	int Find(CDXTileAnim* pAnim);
	int Find(CDXTileViewer* pViewer);
	int Find(CDXTileUnit* pUnit);
	int Find(CDXTileStack* pStack);
	//int Find(CDXTileEngine* pEngine);
	int GetTileLayerCount();
	CDXTileLayer* GetTileLayer(int index);
	int Find(CDXTileLayer* pLayer);
	void DeleteTileLayer(int index);

	CDXTileAnim* GetTileAnim(int index);
	int GetTileAnimCount();
	CDXTileViewer* GetTileViewer(int index);
	int GetTileViewerCount();

	//int GetTileEngineCount();
	//CDXTileEngine* GetTileEngine(int index);
	int GetTileStackCount();
	CDXTileStack* GetTileStack(int index);
	int GetTileUnitCount();
	CDXTileUnit* GetTileUnit(int index);
	CDXTileManager();
	virtual ~CDXTileManager();


	// Shortcut
	CDXTileLayer* Reg(const char* name, CDXTileLayer* pLayer)
		{ return Register(name, pLayer); }
	CDXTileAnim* Reg(const char* name, CDXTileAnim* pAnim)
		{ return Register(name, pAnim); }
	CDXTileViewer* Reg(const char* name, CDXTileViewer* pViewer)
		{ return Register(name, pViewer); }
	CDXTileUnit* Reg(const char* name, CDXTileUnit* pUnit)
		{ return Register(name, pUnit); }
	CDXTileStack* Reg(const char* name, CDXTileStack* pStack)
		{ return Register(name, pStack); }

	CDXSprite* Spr(const char* name)
		{ return Sprite(name); }
	CDXTileLayer* TL(const char* name)
		{ return TileLayer(name); }
	CDXTileAnim* TA(const char* name)
		{ return TileAnim(name); }
	CDXTileViewer* TV(const char* name)
		{ return TileViewer(name); }
	CDXTileUnit* TU(const char* name)
		{ return TileUnit(name); }
	CDXTileStack* TS(const char* name)
		{ return TileStack(name); }

protected:
	CPtrArray m_TileViewers;
	//CPtrArray m_TileEngines;
	CPtrArray m_TileLayers;
	CPtrArray m_TileStacks;
	CPtrArray m_TileUnits;
	CPtrArray m_TileAnims;
	CPtrArray m_CDXTiles;		// CDXTile
	CPtrArray m_CDXSprites;		// CDXSprite

	CMapStringToPtr m_TileViewerMap;
	//CMapStringToPtr m_TileEngineMap
	CMapStringToPtr m_TileLayerMap;
	CMapStringToPtr m_TileStackMap;
	CMapStringToPtr m_TileUnitMap;
	CMapStringToPtr m_TileAnimMap;
	CMapStringToPtr m_CDXTileMap;
	CMapStringToPtr m_CDXSpriteMap;
		
private:
	CDXSprite* Register(const char *name, CDXSprite* pSprite);
	CDXTile* Register(const char* name, CDXTile* pTile);
};

#endif // !defined(AFX_CDXTILEMANAGER_H__3DD28F41_FCC5_11CF_A2B6_82A0EDBA634B__INCLUDED_)
