// CDXTileViewer.cpp: implementation of the CDXTileViewer class.
//
//////////////////////////////////////////////////////////////////////

#include "CDXTileViewer.h"
#include "CDXTileEngine.h"
#include "CDXTileLayer.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDXTileViewer::CDXTileViewer()
{
	Init();
}

CDXTileViewer::CDXTileViewer(CDXSurface* pSurface)
{
	Init();
	SetSurface(pSurface);
}

CDXTileViewer::~CDXTileViewer()
{
	if (m_ViewRect)
		delete m_ViewRect;
}

void CDXTileViewer::Draw(CDXTileEngine *pEngine)
{
	int count = pEngine->GetLayerCount();
	for (int i=0; i<count; i++)
	{
		// Layer can be NULL
		CDXTileLayer* pLayer = pEngine->GetLayer(i);
		if (pLayer)
			pLayer->Draw();
	}
	// it will call each layer drawing routine which
	// will then call each layer's active viewer (which
	// may be different with this viewer)
}

void CDXTileViewer::SetSurface(CDXSurface *pSurface)
{
	m_pSurface = pSurface;
}

void CDXTileViewer::Init()
{
	m_pSurface = 0;
	m_ViewRect = 0;
	SetWrapView(TRUE);
}

void CDXTileViewer::SetWrapView(BOOL bWrap)
{
	SetFlag(BIT_WRAPVIEW, bWrap);
	/*if (bWrap)
		m_InternalFlags |= BIT_WRAPVIEW;
	else
		m_InternalFlags &= ~BIT_WRAPVIEW;*/
}

void CDXTileViewer::SetViewRect(RECT viewRect)
{
	if (m_ViewRect == 0)
		m_ViewRect = new RECT;
	*m_ViewRect = viewRect;
}

BOOL CDXTileViewer::GetViewRect(RECT& viewRect)
{
	if (m_ViewRect == 0)
		return FALSE;
	viewRect = *m_ViewRect;
	return TRUE;
}
