/////////////////////////////////////////////////////////////////////
// CTETutor.cpp : Defines the entry point for the application.
// Step	02		: Creating a simple isometric view scroller
// Author		: Petrus Abri Santoso	09/10/99
/////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
// Generated by the CDX Application Wizard
// By: Michael Rich [istan@alltel.net]
//
// With valuable help from:
//   Ioannis Karagiorgos, for the debugging help
//   Bil Simser, for the good ideas and constructive criticism
/////////////////////////////////////////////////////////////////////

#include "resource.h"

#define CTEINCLUDEALL		// this define includes all headers, otherwise include one by one
#include <cte.h>

/////////////////////////////////////////////////////////////////////
// Methods
/////////////////////////////////////////////////////////////////////
void CreateResources();
void CleanUp();
void ProcessInput();

/////////////////////////////////////////////////////////////////////
// Global Variables
/////////////////////////////////////////////////////////////////////
char szAppName[] = "CTETutor - Creating a simple isometric view scroller";
char szClassName[] = "CTETutorWndClass";

HINSTANCE g_hInst;			// instance handle
HWND g_hWnd;				// window handle

// running fullscreen?
BOOL g_bFullScreen = 
TRUE; 
//FALSE;

/////////////////////////////////////////////////////////////////////
// CDX Objects
/////////////////////////////////////////////////////////////////////
CDXScreen *Screen = 0;
CDXTile *pGroundTile = 0;
CDXSprite *pGroundSpr = 0;

/////////////////////////////////////////////////////////////////////
// CTE Objects
/////////////////////////////////////////////////////////////////////
// The Engine
CDXTileEngine* pEngine = 0;

// Tile units
CDXTileUnit *pGrassUnit = 0;
CDXTileUnit *pMudUnit = 0; 
CDXTileUnit *pWaterUnit = 0;

// Tile Stacks
CDXTileStack *pGrassStack = 0;
CDXTileStack *pMudStack = 0;
CDXTileStack *pWaterStack = 0;

// Viewer
CDXTileIsoViewer *pViewer = 0;

/////////////////////////////////////////////////////////////////////
// Forward declarations
/////////////////////////////////////////////////////////////////////
void AdjustWinStyle();

/////////////////////////////////////////////////////////////////////
// cdx_Init - handles initialization of the CDX objects
/////////////////////////////////////////////////////////////////////
BOOL cdx_Init()
{
	// Create the CDXSreen object
	Screen = new CDXScreen();
	if (Screen==NULL) return FALSE;

	if (g_bFullScreen)
	{
		if (!Screen->CreateFullScreen(g_hWnd, 640, 480, 8)) return FALSE;
	} else
		if (!Screen->CreateWindowed(g_hWnd, 640, 480)) return FALSE;

	// use the palette as defined in the splash screen
	Screen->LoadPalette("TilesN.BMP");

	// TODO: Initialize your own CDX objects here
	CreateResources();
	
	return TRUE;
}

/////////////////////////////////////////////////////////////////////
// cdx_DeInit - handles cleanup of CDX objects
/////////////////////////////////////////////////////////////////////
void cdx_DeInit()
{
	// TODO: Destroy your CDX objects here
	SAFEDELETE(Screen);

	// This will destroy everything
	CleanUp();
}

/////////////////////////////////////////////////////////////////////
// cdx_DoFrame - performs drawing of the current frame
/////////////////////////////////////////////////////////////////////
void cdx_DoFrame()
{

	// clear the current frame
	//Screen->GetBack()->Fill(0);

	ProcessInput();

	// TODO: Add code to draw your objects during each frame
	pEngine->Draw();

	Screen->Flip();	 // Flip the back buffer to the front
}

/////////////////////////////////////////////////////////////////////
// AdjustWinStyle - adjusts the window style according to the mode
/////////////////////////////////////////////////////////////////////
static void AdjustWinStyle()
{
	if (g_bFullScreen)
	{
		DWORD       dwStyle;

		// Change window attributes

		dwStyle = WS_POPUP | WS_VISIBLE;

		SetWindowLong(g_hWnd, GWL_STYLE, dwStyle);

		SetWindowPos(g_hWnd, HWND_TOPMOST, 0, 0, GetSystemMetrics(SM_CXFULLSCREEN),
			GetSystemMetrics(SM_CYFULLSCREEN), SWP_NOACTIVATE |
			SWP_NOZORDER);
	}
	else
	{
		RECT        rect = {0, 0, 640, 480};
		DWORD       dwStyle;

		// Change window attributes

		dwStyle = GetWindowStyle(g_hWnd);
		dwStyle &= ~WS_POPUP;
		dwStyle |= WS_OVERLAPPED | WS_CAPTION | WS_MINIMIZEBOX | WS_SYSMENU;

		SetWindowLong(g_hWnd, GWL_STYLE, dwStyle);

		// Resize the window so that the client area is 640x480

		AdjustWindowRectEx(&rect, GetWindowStyle(g_hWnd), GetMenu(g_hWnd) != NULL,
			GetWindowExStyle(g_hWnd));

		// Just in case the window was moved off the visible area of the
		// screen.

		SetWindowPos(g_hWnd, NULL, 0, 0, rect.right-rect.left,
			rect.bottom-rect.top, SWP_NOMOVE | SWP_NOZORDER |
			SWP_NOACTIVATE);

		SetWindowPos(g_hWnd, HWND_NOTOPMOST, 0, 0, 0, 0,
			SWP_NOSIZE | SWP_NOMOVE | SWP_NOACTIVATE);
	}
}

/////////////////////////////////////////////////////////////////////
// WinProc - handles application messages
/////////////////////////////////////////////////////////////////////
static long PASCAL WinProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{

		case WM_KEYDOWN:
			switch(wParam)
			{
			case VK_ESCAPE:
				// if Escape is pressed, close the application
				SendMessage(hWnd, WM_CLOSE, 0, 0);
				break;
			}
			return  0;
		case WM_SYSKEYUP:
			// Alt+Enter switches modes
			if (wParam == VK_RETURN)
			{
				g_bFullScreen = !g_bFullScreen;
				if(g_bFullScreen)
				{
					AdjustWinStyle();
					// we need to destroy and recreate the supporting CDX objects
					// that can't survive intact when we switch back to fullscreen mode
					{
						// TODO: add your own CDX objects that would have to be destroyed
						// on a mode switch (they will be recreated below)
						CleanUp();

					}
					Screen->CreateFullScreen(g_hWnd, 640, 480, 8);
					{
						// TODO: recreate your CDX objects that were destroyed above

						// load the palette back up
						Screen->LoadPalette("TilesN.BMP");
						CreateResources();
					}
				}
				else
				{
					Screen->GetDD()->RestoreDisplayMode();
					AdjustWinStyle();
					// we need to destroy and recreate the supporting CDX objects
					// that can't survive intact when we switch back to windowed mode
					{
						// TODO: add your own CDX objects that would have to be destroyed
						// on a mode switch (they will be recreated below)
						CleanUp();
					}
					Screen->CreateWindowed(g_hWnd, 640, 480);
					{
						// TODO: recreate your CDX objects that were destroyed above

						// load the palette back up
						Screen->LoadPalette("TilesN.BMP");
						CreateResources();
					}

				}
				return 0;
			}
			return 1;
		case WM_CLOSE:
			cdx_DeInit();
		case WM_DESTROY:
			PostQuitMessage(0);
		return 0;

		default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
}

/////////////////////////////////////////////////////////////////////
// ChangeToEXEDir - sets CWD to the DIR the EXE is in
/////////////////////////////////////////////////////////////////////
static void ChangeToEXEDir()
{
	char buf[MAX_PATH];
	char *cptr;

	//now change the directory
	//to make sure were accessing the proper file
	GetModuleFileName(NULL, buf, MAX_PATH);

	//move pointer to end of string
	cptr = buf + lstrlen(buf);

	//find the end of the path
	do
	{
		cptr--;
	} while (*cptr != '\\');
	cptr++;
	*cptr='\0';

	//change directory
	SetCurrentDirectory(buf);
}

/////////////////////////////////////////////////////////////////////
// InitApp - Create the window and the CDX objects
/////////////////////////////////////////////////////////////////////
static BOOL InitApp(int nCmdShow)
{
	WNDCLASS WndClass;

	WndClass.style = CS_HREDRAW | CS_VREDRAW;
	WndClass.lpfnWndProc = WinProc;
	WndClass.cbClsExtra = 0;
	WndClass.cbWndExtra = 0;
	WndClass.hInstance = g_hInst;
	WndClass.hIcon = LoadIcon(g_hInst, "APPICON");
	WndClass.hCursor = LoadCursor(0, IDC_ARROW);
	WndClass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	WndClass.lpszMenuName = 0;
	WndClass.lpszClassName = szClassName;
	RegisterClass(&WndClass);

	g_hWnd = CreateWindowEx(
		0,
		szClassName,
		szAppName,
 		WS_OVERLAPPED | WS_CAPTION | WS_MINIMIZEBOX | WS_SYSMENU,
		0,
		0,
		320,
		240,
		NULL,
		NULL,
		g_hInst,
		NULL);

	if(!g_hWnd) return FALSE;

	AdjustWinStyle();

	ShowWindow(g_hWnd, nCmdShow);
	UpdateWindow(g_hWnd);

	ChangeToEXEDir();
	
	return TRUE;
}

/////////////////////////////////////////////////////////////////////
// WinMain - inital function called by windows
/////////////////////////////////////////////////////////////////////
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nCmdShow)
{
	MSG msg;

	// save the app instance
	g_hInst = hInstance;


	if(!InitApp(nCmdShow)) return FALSE;
	if(!cdx_Init())
	{
		PostQuitMessage(0);
		return FALSE;
	}

	while(1)
	{
		if(PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if(!GetMessage(&msg, NULL, 0, 0 )) return msg.wParam;
			TranslateMessage(&msg); 
			DispatchMessage(&msg);
		}
		else
		{
			cdx_DoFrame();
		}
	}
}

void CreateResources()
{
	// Create the Engine
	pEngine = new CDXTileEngine();
	
	// Layer Creation
	CDXTileLayer* pLayer = pEngine->CreateLayer(30, 30, 32, 16);

	// Bitmap Loading
	pGroundTile = new CDXTile(Screen, "TilesN.BMP", 32, 16, 50);
	pGroundSpr = new CDXSprite(pGroundTile);
	pGroundSpr->SetColorKey(0);

	// Tiles Creation
	pGrassUnit = new CDXTileUnit(pGroundSpr, 0, 0);
	pMudUnit = new CDXTileUnit(pGroundSpr, 1, 0);
	pWaterUnit = new CDXTileUnit(pGroundSpr, 2, 0);

	// Stacks Creation
	pGrassStack = new CDXTileStack();
	pGrassStack->AddTile(pGrassUnit);
	pMudStack = new CDXTileStack();
	pMudStack->AddTile(pMudUnit);
	pWaterStack = new CDXTileStack();
	pWaterStack->AddTile(pWaterUnit);

	// World Creation
	for (int mapY=0; mapY<30; mapY++)
		for (int mapX=0; mapX<30; mapX++)
		{
			CDXTileStack* pStack;
			int unitIndex = (mapY*mapX) % 3;
			switch (unitIndex)
			{
				case 0: pStack = pGrassStack; break;
				case 1: pStack = pMudStack; break;
				case 2: pStack = pWaterStack; break;
			}
			pLayer->SetTileStack(mapX, mapY, pStack);
		}

	// Set the viewer
	pViewer = new CDXTileIsoViewer(Screen->GetBack());
	pEngine->SetViewer(pViewer);
}

void ProcessInput()
{
#define scroll_speed 4
	if(GetAsyncKeyState(VK_UP)) 
	{
		pEngine->ScrollMap(0, -scroll_speed);
	} else
	if(GetAsyncKeyState(VK_DOWN)) 
	{
		pEngine->ScrollMap(0, +scroll_speed);
	} else
	if(GetAsyncKeyState(VK_LEFT))
	{
		pEngine->ScrollMap(-scroll_speed*2, 0);
	} else
	if(GetAsyncKeyState(VK_RIGHT)) 
	{
		pEngine->ScrollMap(+scroll_speed*2, 0);
	}
}

void CleanUp()
{
	// CDXTile & CDXSprite
	SAFEDELETE(pGroundTile);
	SAFEDELETE(pGroundSpr);
	
	// Tile Units
	SAFEDELETE(pGrassUnit);
	SAFEDELETE(pMudUnit);
	SAFEDELETE(pWaterUnit);

	// Tile Stacks
	SAFEDELETE(pGrassStack);
	SAFEDELETE(pMudStack);
	SAFEDELETE(pWaterStack);

	// Viewer & engine
	SAFEDELETE(pViewer);
	SAFEDELETE(pEngine);
}